@@ -1,10 +1,10 @@
 import itertools
 import json
 import demjson3 as demjson
+from demjson3 import JSONDecodeError
 import logging
 import re
 from datetime import datetime
-from json import JSONDecodeError
 from typing import Any, Dict, Optional
 from urllib.parse import parse_qs, urlparse
 from tqdm.auto import tqdm
@@ -133,6 +133,8 @@ class PostExtractor:
             'reactions': None,
             'reaction_count': None,
             'with': None,
+            'page_id': None,
+            'sharers': None,
         }
 
     def extract_post(self) -> Post:
@@ -189,6 +191,9 @@ class PostExtractor:
         if self.options.get('reactions') or self.options.get('reactors'):
             try:
                 reactions = self.extract_reactions()
+                if reactions["reactors"] and self.options.get("reactors") != "generator":
+                    # Consume reactor generator to return list
+                    reactions["reactors"] = utils.safe_consume(reactions["reactors"])
             except Exception as ex:
                 log_warning("Exception while extracting reactions: %r", ex)
                 reactions = {}
@@ -198,6 +203,14 @@ class PostExtractor:
             else:
                 post.update(reactions)
 
+        if self.options.get("sharers"):
+            try:
+                post["sharers"] = self.extract_sharers()
+                if self.options.get("sharers") != "generator":
+                    post["sharers"] = utils.safe_consume(post["sharers"])
+            except Exception as ex:
+                log_warning("Exception while extracting sharers: %r", ex)
+
         if self.options.get('comments'):
             try:
                 post["comments_full"] = self.extract_comments_full()
@@ -206,6 +219,11 @@ class PostExtractor:
                     post["comments_full"] = utils.safe_consume(post["comments_full"])
                     for comment in post["comments_full"]:
                         comment["replies"] = utils.safe_consume(comment["replies"])
+                        for reply in comment["replies"]:
+                            utils.safe_consume(reply["comment_reactors"])
+                        comment["comment_reactors"] = utils.safe_consume(
+                            comment["comment_reactors"]
+                        )
                     if post.get("comments_full") and not post.get("comments"):
                         post["comments"] = len(post.get("comments_full"))
 
@@ -324,13 +342,17 @@ class PostExtractor:
             return None
 
     def extract_user_id(self) -> PartialPost:
-        return {'user_id': self.data_ft['content_owner_id_new']}
+        return {
+            'user_id': self.data_ft['content_owner_id_new'],
+            'page_id': self.data_ft.get("page_id"),
+        }
 
     def extract_image_lq(self) -> PartialPost:
         elems = self.element.find('div.story_body_container>div .img:not(.profpic)')
         if not elems:
             elems = self.element.find('.img:not(.profpic), img:not(.profpic)')
         images = []
+        image_ids = []
         descriptions = []
         for elem in elems:
             url = None
@@ -343,10 +365,16 @@ class PostExtractor:
             if url and "static.xx.fbcdn.net" not in url:
                 images.append(url)
                 descriptions.append(elem.attrs.get("aria-label") or elem.attrs.get("alt"))
+            url = elem.element.getparent().getparent().getparent().attrib.get("href")
+            if url:
+                image_ids.append(re.search(r'[=/](\d+)', url).group(1))
 
         image = images[0] if images else None
+        image_id = image_ids[0] if image_ids else None
         result = {
             "image_lowquality": image,
+            "image_id": image_id,
+            "image_ids": image_ids,
             "images_lowquality": images,
             "images_lowquality_description": descriptions,
         }
@@ -473,12 +501,18 @@ class PostExtractor:
         images = []
         descriptions = []
         image_ids = []
-        photo_links = self.element.find(
+        raw_photo_links = self.element.find(
             "div.story_body_container>div a[href*='photo.php'], "
             "div.story_body_container>div a[href*='/photos/'], "
             "div._5v64 a[href*='/photos/']"
         )
-
+        photo_links = []
+        seen_urls = []
+        for a in raw_photo_links:
+            partial_url = a.attrs["href"].split("?")[0]
+            if partial_url not in seen_urls:
+                photo_links.append(a)
+                seen_urls.append(partial_url)
         total_photos_in_gallery = len(photo_links)
         if len(photo_links) in [4, 5] and photo_links[-1].text:
             total_photos_in_gallery = len(photo_links) + int(photo_links[-1].text.strip("+")) - 1
@@ -571,6 +605,96 @@ class PostExtractor:
             "image_ids": image_ids,
         }
 
+    def extract_reactors(self, response, reaction_lookup):
+        """Fetch people reacting to an existing post obtained by `get_posts`.
+        Note that this method may raise one more http request per post to get all reactors"""
+        emoji_class_lookup = {}
+        spriteMapCssClass = "sp_E24l_TeOlgh"
+        for k, v in self.get_jsmod("UFIReactionIcons").items():
+            name = reaction_lookup[k]["display_name"].lower()
+            for item in v.values():
+                emoji_class_lookup[item["spriteCssClass"]] = name
+                spriteMapCssClass = item["spriteMapCssClass"]
+
+        reactors_opt = self.options.get("reactors")
+        limit = 3000
+        if type(reactors_opt) in [int, float] and reactors_opt < limit:
+            limit = reactors_opt
+        logger.debug(f"Fetching {limit} reactors")
+        elems = list(response.html.find("div[id^='reaction_profile_browser']>div"))
+        for elem in elems:
+            emoji_class = elem.find(f"div>i.{spriteMapCssClass}", first=True).attrs.get("class")[
+                -1
+            ]
+            if not emoji_class_lookup.get(emoji_class):
+                logger.error(f"Don't know {emoji_class}")
+            yield {
+                "name": elem.find("strong", first=True).text,
+                "link": utils.urljoin(FB_BASE_URL, elem.find("a", first=True).attrs.get("href")),
+                "type": emoji_class_lookup.get(emoji_class),
+            }
+        more = response.html.find("div[id^=reaction_profile_pager] a", first=True)
+        while more and len(elems) < limit:
+            url = utils.urljoin(FB_MOBILE_BASE_URL, more.attrs.get("href"))
+            logger.debug(f"Fetching {url}")
+            try:
+                response = self.request(url)
+            except Exception as e:
+                logger.error(e)
+                break
+            prefix_length = len('for (;;);')
+            data = json.loads(response.text[prefix_length:])  # Strip 'for (;;);'
+            more = None
+            for action in data['payload']['actions']:
+                if action['cmd'] == 'append':
+                    html = utils.make_html_element(
+                        f"<div id='reaction_profile_browser'>{action['html']}</div>",
+                        url=FB_MOBILE_BASE_URL,
+                    )
+                    elems = html.find(
+                        'div#reaction_profile_browser>div,div#reaction_profile_browser1>div'
+                    )
+                    for elem in elems:
+                        emoji_class = elem.find(
+                            f"div>i.{spriteMapCssClass}", first=True
+                        ).attrs.get("class")[-1]
+                        if not emoji_class_lookup.get(emoji_class):
+                            logger.error(f"Don't know {emoji_class}")
+                        yield {
+                            "name": elem.find("strong", first=True).text,
+                            "link": utils.urljoin(
+                                FB_BASE_URL, elem.find("a", first=True).attrs.get("href")
+                            ),
+                            "type": emoji_class_lookup.get(emoji_class),
+                        }
+                elif action['cmd'] == 'replace':
+                    html = utils.make_html_element(
+                        f"<div id='reaction_profile_browser'>{action['html']}</div>",
+                        url=FB_MOBILE_BASE_URL,
+                    )
+                    more = html.find("div#reaction_profile_pager a", first=True)
+
+    def extract_sharers(self):
+        """Fetch people sharing an existing post obtained by `get_posts`.
+        Note that this method may raise more http requests per post to get all sharers"""
+        share_url = f'https://m.facebook.com/browse/shares?id={self.post.get("post_id")}'
+        while share_url:
+            logger.debug(f"Fetching {share_url}")
+            response = self.request(share_url)
+            elems = response.html.find("div.item:not(#m_more_item)")
+            for elem in elems:
+                yield {
+                    "name": elem.find("strong", first=True).text,
+                    "link": utils.urljoin(
+                        FB_BASE_URL, elem.find("a", first=True).attrs.get("href")
+                    ),
+                }
+            more = response.html.find("#m_more_item a", first=True)
+            if more:
+                share_url = more.attrs.get("href")
+            else:
+                share_url = None
+
     def extract_reactions(self, post_id=None) -> PartialPost:
         """Fetch share and reactions information with a existing post obtained by `get_posts`.
         Return a merged post that has some new fields including `reactions`, `w3_fb_url`,
@@ -602,85 +726,26 @@ class PostExtractor:
             post_id = self.post.get("post_id")
         w3_fb_url = url and utils.urlparse(url)._replace(netloc='www.facebook.com').geturl()
 
-        reactors = []
         reactors_opt = self.options.get("reactors")
+        reactors = []
         if reactors_opt:
             reaction_url = f'https://m.facebook.com/ufi/reaction/profile/browser/?ft_ent_identifier={post_id}'
             logger.debug(f"Fetching {reaction_url}")
             response = self.request(reaction_url)
 
-            reactions = {}
-            for sigil in response.html.find("span[data-sigil='reaction_profile_sigil']"):
-                k = str(demjson.decode(sigil.attrs.get("data-store"))["reactionType"])
-                v = sigil.find(
-                    "span[data-sigil='reaction_profile_tab_count']", first=True
-                ).text.replace("All ", "")
-                v = utils.convert_numeric_abbr(v)
-                if k == "all":
-                    reaction_count = v
-                elif k in reaction_lookup:
-                    name = reaction_lookup[k]["display_name"].lower()
-                    reactions[name] = v
-
-            emoji_class_lookup = {}
-            spriteMapCssClass = "sp_E24l_TeOlgh"
-            for k, v in self.get_jsmod("UFIReactionIcons").items():
-                name = reaction_lookup[k]["display_name"].lower()
-                for item in v.values():
-                    emoji_class_lookup[item["spriteCssClass"]] = name
-                    spriteMapCssClass = item["spriteMapCssClass"]
-
-            """Fetch people reacting to an existing post obtained by `get_posts`.
-            Note that this method may raise one more http request per post to get all reactors"""
-            limit = 3000
-            if type(reactors_opt) in [int, float] and reactors_opt < limit:
-                limit = reactors_opt
-            logger.debug(f"Fetching {limit} reactors")
-            elems = list(response.html.find("div[id^='reaction_profile_browser']>div"))
-            more = response.html.find("div#reaction_profile_pager a", first=True)
-            while more and len(elems) < limit:
-                url = utils.urljoin(FB_MOBILE_BASE_URL, more.attrs.get("href"))
-                logger.debug(f"Fetching {url}")
-                try:
-                    response = self.request(url)
-                except Exception as e:
-                    logger.error(e)
-                    break
-                prefix_length = len('for (;;);')
-                data = json.loads(response.text[prefix_length:])  # Strip 'for (;;);'
-
-                for action in data['payload']['actions']:
-                    if action['cmd'] == 'append':
-                        html = utils.make_html_element(
-                            f"<div id='reaction_profile_browser'>{action['html']}</div>",
-                            url=FB_MOBILE_BASE_URL,
-                        )
-                        more_elems = html.find(
-                            'div#reaction_profile_browser>div,div#reaction_profile_browser1>div'
-                        )
-                        elems.extend(more_elems)
-                    elif action['cmd'] == 'replace':
-                        html = utils.make_html_element(
-                            f"<div id='reaction_profile_browser'>{action['html']}</div>",
-                            url=FB_MOBILE_BASE_URL,
-                        )
-                        more = html.find("div#reaction_profile_pager a", first=True)
-            logger.debug(f"Found {len(elems)} reactors")
-            for elem in elems:
-                emoji_class = elem.find(f"div>i.{spriteMapCssClass}", first=True).attrs.get(
-                    "class"
-                )[-1]
-                if not emoji_class_lookup.get(emoji_class):
-                    logger.error(f"Don't know {emoji_class}")
-                reactors.append(
-                    {
-                        "name": elem.find("strong", first=True).text,
-                        "link": utils.urljoin(
-                            FB_BASE_URL, elem.find("a", first=True).attrs.get("href")
-                        ),
-                        "type": emoji_class_lookup.get(emoji_class),
-                    }
-                )
+            if not reactions:
+                for sigil in response.html.find("span[data-sigil='reaction_profile_sigil']"):
+                    k = str(demjson.decode(sigil.attrs.get("data-store"))["reactionType"])
+                    v = sigil.find(
+                        "span[data-sigil='reaction_profile_tab_count']", first=True
+                    ).text.replace("All ", "")
+                    v = utils.convert_numeric_abbr(v)
+                    if k == "all":
+                        reaction_count = v
+                    elif k in reaction_lookup:
+                        name = reaction_lookup[k]["display_name"].lower()
+                        reactions[name] = v
+            reactors = self.extract_reactors(response, reaction_lookup)
 
         if reactions:
             return {
@@ -912,13 +977,18 @@ class PostExtractor:
                     image_url = utils.decode_css_url(match.groups()[0])
 
         reactions = {}
-        if self.options.get("reactions") or self.options.get("reactors"):
+        comment_reactors_opt = self.options.get(
+            "comment_reactors", self.options.get("reactions") or self.options.get("reactors")
+        )
+        if comment_reactors_opt:
             self.options["reactors"] = True  # Required for comment reaction extraction
             reactors = comment.find(
                 'a[href^="/ufi/reaction/profile/browser/?ft_ent_identifier="] i', first=True
             )
             if reactors:
                 reactions = self.extract_reactions(comment_id)
+                if comment_reactors_opt != "generator":
+                    reactions["reactors"] = utils.safe_consume(reactions.get("reactors", []))
 
         return {
             "comment_id": comment_id,
@@ -930,7 +1000,7 @@ class PostExtractor:
             "comment_text": text,
             "comment_time": date,
             "comment_image": image_url,
-            "comment_reactors": reactions.get("reactors"),
+            "comment_reactors": reactions.get("reactors", []),
             "comment_reactions": reactions.get("reactions"),
             "comment_reaction_count": reactions.get("reaction_count"),
         }
@@ -1039,7 +1109,7 @@ class PostExtractor:
 
         while more_url and len(comments) <= limit:
             if request_url_callback:
-                request_url_callback(more_url)
+                request_url_callback(utils.urljoin(FB_MOBILE_BASE_URL, more_url))
             if more_url in visited_urls:
                 logger.debug("cycle detected, break")
                 break
@@ -1113,7 +1183,7 @@ class PostExtractor:
 
         self._data_ft = {}
         try:
-            data_ft_json = self.element.attrs['data-ft']
+            data_ft_json = self.element.attrs['data-ft'].replace("\\\\", "\\")
             self._data_ft = demjson.decode(data_ft_json)
         except JSONDecodeError as ex:
             logger.error("Error parsing data-ft JSON: %r", ex)
