@@ -12,6 +12,12 @@ from requests.cookies import RequestsCookieJar
 from requests_html import DEFAULT_URL, Element, PyQuery
 import json
 
+from . import exceptions
+import logging
+import time
+
+logger = logging.getLogger(__name__)
+
 
 def find_and_search(node, selector, pattern, cast=str):
     container = node.find(selector, first=True)
@@ -178,8 +184,6 @@ def parse_datetime(text: str, search=True) -> Optional[datetime]:
             if text == today:
                 # Fix for dateparser misinterpreting "last Monday" as today if today is Monday
                 return dateparser.parse(text) - timedelta(days=7)
-        else:
-            return None
 
     result = dateparser.parse(text)
     if result:
@@ -247,3 +251,14 @@ def parse_cookie_file(filename: str) -> RequestsCookieJar:
             jar.set(name, value, domain=domain, path=path, secure=secure, expires=expires)
 
     return jar
+
+
+def safe_consume(generator, sleep=0):
+    result = []
+    try:
+        for item in generator:
+            result.append(item)
+            time.sleep(sleep)
+    except Exception as e:
+        logger.error(f"Exception when consuming {generator}: {type(e)}: {str(e)}")
+    return result
